import * as fs from 'node:fs';
import { resolve } from 'node:path';
import { react, reactRefresh } from './presets/react';
import { tailwind } from './presets/tailwind';
import { jsdoc } from './presets/jsdoc';
import { storybook, reactQuery } from './presets/misc';
import { testingLibrary } from './presets/testing-library';
import type { Middleware } from './middleware';

const middlewares = {
  react,
  reactRefresh,
  tailwind,
  storybook,
  reactQuery,
  testingLibrary,
  jsdoc,
};

export const envs: {
  dependency: string;
  eslintPlugin?: string;
  middleware: keyof typeof middlewares;
}[] = [
  {
    dependency: 'react',
    middleware: 'react',
  },
  {
    dependency: '@vitejs/plugin-react',
    eslintPlugin: 'eslint-plugin-react-refresh',
    middleware: 'reactRefresh',
  },
  {
    dependency: 'tailwindcss',
    eslintPlugin: 'eslint-plugin-tailwindcss',
    middleware: 'tailwind',
  },
  {
    dependency: 'storybook',
    eslintPlugin: 'eslint-plugin-storybook',
    middleware: 'storybook',
  },
  {
    dependency: '@tanstack/react-query',
    eslintPlugin: '@tanstack/eslint-plugin-query',
    middleware: 'reactQuery',
  },
  {
    dependency: '@testing-library/react',
    eslintPlugin: 'eslint-plugin-testing-library',
    middleware: 'testingLibrary',
  },
];

export function getProjectDependencies() {
  const rootDir = process.cwd();

  const pkgJsonPath = resolve(rootDir, 'package.json');
  const pkgJson = fs.existsSync(pkgJsonPath)
    ? JSON.parse(fs.readFileSync(pkgJsonPath, 'utf-8'))
    : {};

  return new Set(
    Object.keys({
      ...pkgJson.dependencies,
      ...pkgJson.devDependencies,
      ...pkgJson.peerDependencies,
    }),
  );
}

export function* checkEnv(): Generator<Middleware> {
  const deps = getProjectDependencies();

  for (const { dependency, eslintPlugin, middleware } of envs) {
    if (deps.has(dependency) && (!eslintPlugin || deps.has(eslintPlugin))) {
      yield middlewares[middleware];
    }
  }
}
